####Function: Map omics data and cell label####
Mapping = function(Data,label_temp)
{
  label_temp=label_temp[-which(label_temp[,8]=="UNABLE TO CLASSIFY"),] 
  label_temp=label_temp[-which(is.na(label_temp[,8])),] 
  label=matrix(data = NA,nrow = ncol(Data),ncol = 1) 
  names(label)=colnames(Data) 
  non_name=NULL
  for (i in 1:ncol(Data))
  {
    index=which(label_temp[,1]==colnames(Data)[i]) 
    if (length(index)>0) 
    {
      label[i]=label_temp[index,8]
    }else 
    {
      non_name=c(non_name,i)
    }
  }
  if(is.null(non_name)) 
  {
    result=list(Data=Data,label=label)
    return(result)
  }else{ 
    Data=Data[,-non_name] 
    label=label[-non_name] 
    result=list(Data=Data,label=label) 
    return(result)
  }
}

####Function：Select optimal cluster number####
OptClustNum=function(Data,Method="hc")
{
  #Input:Data matrix. Row represents feature, and column represents samples.
  #Input:Method,"hc".
  #Output:Optimal cluster number.
  library(NbClust)
  seed=8715
  index=c("kl", "ch", "hartigan", "ccc", "scott", "marriot", "trcovw", "tracew", "friedman", "rubin", "cindex", "db", "silhouette", "duda", "pseudot2", "beale", "ratkowsky", "ball", "ptbiserial", "gap", "frey", "mcclain", "dunn", "hubert", "sdindex", "dindex", "sdbw")
  results=NULL
  for(i in 1:length(index))
  {
    temp=try(NbClust(t(Data), min.nc=10, max.nc=30, method="average", index=index[i]),silent = TRUE) 
    
    if('try-error' %in% class(temp))
    {
      next
    }else{
      results=c(results,temp$Best.nc[1]) 
    }
  }
  temp=as.matrix(table(results)) 
  optimal_cluster_num=max(as.numeric(rownames(temp)[which.max(temp)])) 
  return(optimal_cluster_num)
}


####Function: Pack cluster result(1,1,1，....2,2,2,2,....3,3,...4....)####
ClusterPack=function(Data,cluster)
{
  #Input:Data matrix. Row represents feature, and column represents samples.
  #Input:Cluster result(cluster_result), whose rank is the same as column names of "Data".
  #Output:Cluster output ordered by clustering assign(1,1...,1,2,2,...,2,3,3,...,3,...).
  temp=matrix(data = cluster,ncol=1) 
  rownames(temp)=colnames(Data) 
  temp=as.matrix(sort(temp[,1],na.last=T)) 
  return(temp)
}

####Function: Select signature####
SignaturePack=function(Data, ClusterPackResult,signature_max=500,signature_min=30)
{
  #Input:Data matrix. Row represents feature, and column represents samples.
  #Input:Clusterpack result(cluster_output), whose rank is like(1,1...,1,2,2,...,2,3,3,...,3,...)
  #Input:max and min signature number 
  #Output:name of signatures
  library(pamr)
  set.seed(120)
  Data = Data[,rownames(ClusterPackResult)] 
  mydata = list(x=Data,y=factor(ClusterPackResult[,1]), geneid=as.character(1:nrow(Data)),
                genenames=rownames(Data))   #reconstruction the data 
  mytrain = pamr.train(mydata) 
  #plot(mytrain$threshold,mytrain$errors)
  #plot(mytrain$nonzero,mytrain$errors)
  mycv = pamr.cv(mytrain,mydata) 
  #plot(mycv$threshold,mycv$error)
  #plot(mycv$size,mycv$error)
  
  mytrain_threshold=mytrain$threshold 
  mytrain_errors=mytrain$errors 
  mycv_error=mycv$error 
  mytrain_nonzero=mytrain$nonzero 
  index_zero_temp=which(mytrain_threshold==0) 
  index_lessten_temp=which(mytrain_nonzero<signature_min) 
  index_all_temp=unique(union(index_zero_temp,index_lessten_temp)) 
  if (length(index_all_temp)>0)
  { 
    mytrain_threshold=mytrain_threshold[-index_all_temp]
    mytrain_errors=mytrain_errors[-index_all_temp]
    mycv_error=mycv_error[-index_all_temp]
    mytrain_nonzero=mytrain_nonzero[-index_all_temp]
  }
  rm(index_zero_temp,index_lessten_temp,index_all_temp)
  temp=mytrain_errors/nrow(Data)+mycv_error 
  threshold = mytrain_threshold[max(which(temp==min(temp),arr.ind=TRUE))]
  rm(temp)
  
  pamrgene_index=pamr.listgenes(mytrain, mydata,fitcv = mycv,threshold = threshold)
  
  
  temp=matrix(as.numeric(pamrgene_index),nrow = nrow(pamrgene_index)) 
  colnames(temp)=colnames(pamrgene_index)
  pamrgene_index=temp[order(temp[,"av-rank-in-CV"],decreasing=F),] 
  
  if (nrow(pamrgene_index)>signature_max) 
  {
    pamrgene_index=pamrgene_index[1:signature_max,] 
  }
  pamrgene_index=pamrgene_index[which(pamrgene_index[,"prop-selected-in-CV"]>=0.99),] 
  
  signature_name=rownames(mydata$x)[pamrgene_index[,1]] 
  #signature_name=mydata$genenames[pamrgene_index[,1]]
  rm(temp)
  if (0)
  {
    library(pheatmap)
    data_filter=mydata$x[pamrgene_index[,1],]
    #pamrgene_index=pamrgene_index[order(pamrgene_index[,2],decreasing = T),]
    hhh=pheatmap(data_filter,cluster_cols=F,cluster_rows = T)
    data_filter=data_filter[hhh$tree_row$order,]
    pheatmap(data_filter,cluster_cols=F,cluster_rows = F)
    signature_name=signature_name[hhh$tree_row$order]
    rm(hhh)
  }
  return(signature_name)
}

####Function: mRNA cluster####
H_cluster_method_mRNA=function(Data, ClusterNum=NULL)
{
  Data_rep=Data #Back up for signature illustration
  Data=t(as.matrix(scale(t(Data)))) #Normalization for feature 
  if (is.null(ClusterNum))
  {
    if (ncol(Data)<=20) 
    {
      cluster_number=2
    }else
    {
      cluster_number=OptClustNum(Data=Data,Method="hc") 
    }
  }else
  {
    cluster_number=ClusterNum
  }
  hc=hclust(as.dist(1-cor(Data)),"ward.D") 
  cluster_result=cutree(hc,k=cluster_number) 
  
  cluster_output=ClusterPack(Data=Data,cluster=cluster_result) 
  
  signature_result=SignaturePack(Data=Data_rep,ClusterPackResult=cluster_output)
  
  result=list(cluster_number=cluster_number,cluster_output=cluster_output,signature_result=signature_result)
  return(result)
}

####Function: miRNA cluster####
H_cluster_method_miRNA=function(Data, ClusterNum=NULL)
{
  Data_rep=Data #Back up for signature illustration
  Data=t(as.matrix(scale(t(Data)))) #Normalization for feature  
  if (is.null(ClusterNum))
  {
    if (ncol(Data)<=20) 
    {
      cluster_number=2
    }else
    {
      cluster_number=OptClustNum(Data=Data,Method="hc") #Determine the optimal cluster number
    }
  }else
  {
    cluster_number=ClusterNum
  }
  hc=hclust(as.dist(1-cor(Data)),method = "ward.D2") 
  
  cluster_result=cutree(hc,k=cluster_number) 
  
  cluster_output=ClusterPack(Data=Data,cluster=cluster_result) 
  
  signature_result=SignaturePack(Data=Data_rep,ClusterPackResult=cluster_output)
  
  result=list(cluster_number=cluster_number,cluster_output=cluster_output,signature_result=signature_result)
  return(result)
}

####Function: CNV cluster####
H_cluster_method_CNV=function(Data, ClusterNum=NULL)
{
  Data_rep=Data #Back up for signature illustration
  Data=t(as.matrix(scale(t(Data)))) #Normalization for feature
  if (is.null(ClusterNum))
  {
    if (ncol(Data)<=20)
    {
      cluster_number=2
    }else
    {
      cluster_number=OptClustNum(Data=Data,Method="hc") #Determine the optimal cluster number
    }
  }else
  {
    cluster_number=ClusterNum
  }
  dist.hc=dist(t(Data),method = "manhattan")
  hc=hclust(dist.hc,method = "ward.D2") 
  cluster_result=cutree(hc,k=cluster_number) 
  
  cluster_output=ClusterPack(Data=Data,cluster=cluster_result) 
  
  signature_result=SignaturePack(Data=Data_rep,ClusterPackResult=cluster_output)
  
  result=list(cluster_number=cluster_number,cluster_output=cluster_output,signature_result=signature_result)
  return(result)
}

####Function: methy cluster####
H_cluster_method_methy=function(Data, ClusterNum=10)
{
  Data_rep=Data #Back up for signature illustration
  Data=t(as.matrix(scale(t(Data)))) #Normalization for feature
  if (is.null(ClusterNum))
  {
    if (ncol(Data)<=20) 
    {
      cluster_number=2
    }else
    {
      cluster_number=OptClustNum(Data=Data,Method="hc") #Determine the optimal cluster number
    }
  }else
  {
    cluster_number=ClusterNum
  }
 
  hc=hclust(as.dist(1-cor(Data)),method = "ward.D")
  cluster_result=cutree(hc,k=cluster_number) 
  
  cluster_output=ClusterPack(Data=Data,cluster=cluster_result) 
  
  signature_result=SignaturePack(Data=Data_rep,ClusterPackResult=cluster_output)
  
  result=list(cluster_number=cluster_number,cluster_output=cluster_output,signature_result=signature_result)
  return(result)
}

####Function: RPPA cluster####
H_cluster_method_RPPA=function(Data, ClusterNum=10)
{
  Data_rep=Data #Back up for signature illustration
  Data=t(as.matrix(scale(t(Data)))) #Normalization for feature
  if (is.null(ClusterNum))
  {
    if (ncol(Data)<=20) 
    {
      cluster_number=2
    }else
    {
      cluster_number=OptClustNum(Data=Data,Method="hc") #Determine the optimal cluster number
    }
  }else
  {
    cluster_number=ClusterNum
  }
  hc=hclust(as.dist(1-cor(Data)),method = "ward.D2") 
  cluster_result=cutree(hc,k=cluster_number)
  
  cluster_output=ClusterPack(Data=Data,cluster=cluster_result) 
  
  signature_result=SignaturePack(Data=Data_rep,ClusterPackResult=cluster_output)
  
  result=list(cluster_number=cluster_number,cluster_output=cluster_output,signature_result=signature_result)
  return(result)
}

####Function: calculate the number of unique TCGA label in each cluster####
Statistic = function(Data,annotation_col)
{
  Data_stat=matrix(data = NA,nrow = 2,ncol = Data)
  rownames(Data_stat) = c("cluster number","label")
  for (i in 1:Data) 
  {
    Data_stat[1,i] = i
    Data_stat[2,i] = length(unique(annotation_col[which(annotation_col[,1]==i),2]))
  }
  return(Data_stat)
}

####Main####
####Read omics data####

####Map the TCGA label and cell line####
mRNA_result = Mapping(mRNA_temp,label_temp)
miRNA_result = Mapping(miRNA_temp,label_temp)
CNV_result = Mapping(CNV_temp,label_temp)
methy_result = Mapping(methy_temp,label_temp)
RPPA_result = Mapping(RPPA_temp,label_temp)

####Cluster####
mRNA_cluster = H_cluster_method_mRNA(mRNA_result$Data)
miRNA_cluster = H_cluster_method_miRNA(miRNA_result$Data)
CNV_cluster = H_cluster_method_CNV(CNV_result$Data)
methy_cluster = H_cluster_method_methy(methy_result$Data)
RPPA_cluster = H_cluster_method_RPPA(RPPA_result$Data)

####Heatmap####
####Color of mRNA_cancer types####
library(RColorBrewer)
color_type_mRNA=c(brewer.pal(9,"Reds")[-c(1,2,9)],brewer.pal(9,"Greens")[-c(1,2,3,7,8,9)],brewer.pal(9,"PuRd")[-c(1,2,7,8,9)],brewer.pal(9,"Blues")[-c(1,2,9)],brewer.pal(9,"Oranges")[-c(1,2,7,8,9)],brewer.pal(9,"Purples")[-c(1,2,3,8,9)],brewer.pal(8,"Set2")[-c(2,3,4,8)])
color_cluster_mRNA=brewer.pal(12,"Set3")[-c(2,9)]
TCGA_label_mRNA = c("ALL","CLL","DLBC","LAML","LCML","MM",
               "BLCA","KIRC","PRAD",
               "BRCA","CESC","OV","UCEC",
               "CHOL","COAD/READ","ESCA","LIHC","PAAD","STAD",
               "GBM","LGG","MB","NB",
               "LUAD","LUSC","MESO","SCLC",
               "HNSC","SARC","SKCM","THCA")
color1=list(mRNA_group=color_cluster_mRNA,TCGA_disease=color_type_mRNA)
names(color1[[1]])=rownames(unique(mRNA_cluster$cluster_output))

mRNA=data.frame(unique(mRNA_result$label))
mRNA_order=factor(unique(mRNA_result$label),levels = TCGA_label_mRNA)
mRNA=mRNA[order(mRNA_order),]
names(color1[[2]])=as.vector(mRNA)

####mRNA Heatmap####
library(pheatmap)
library(RColorBrewer)
annotation_col_mRNA = data.frame(mRNA_group = mRNA_cluster$cluster_output,TCGA_disease=mRNA_result$label[rownames(mRNA_cluster$cluster_output)])
rownames(annotation_col_mRNA) = rownames(mRNA_cluster$cluster_output)
annotation_col_mRNA = annotation_col_mRNA[order(annotation_col_mRNA[,1],annotation_col_mRNA[,2]),]
mRNA_result$Data[which(mRNA_result$Data>10)]=10
pheatmap(mRNA_result$Data[mRNA_cluster$signature_result,rownames(annotation_col_mRNA)],cluster_cols = F,cluster_rows = T,
         annotation_col = annotation_col_mRNA,show_colnames = F,show_rownames = F,
         annotation_colors=color1,
         color = colorRampPalette(colors = c("blue","white","red"))(100))

####Color of miRNA_cancer types####
library(RColorBrewer)
color_type_miRNA=c(brewer.pal(9,"Reds")[-c(1,2,9)],brewer.pal(9,"Greens")[-c(1,2,3,7,8,9)],brewer.pal(9,"PuRd")[-c(1,2,4,7,8,9)],brewer.pal(9,"Blues")[-c(1,2,9)],brewer.pal(9,"Oranges")[-c(1,2,7,8,9)],brewer.pal(9,"Purples")[-c(1,2,3,8,9)],brewer.pal(8,"Set2")[-c(2,3,4,8)])
color_cluster_miRNA=brewer.pal(12,"Set3")[-c(2,9)]
TCGA_label_miRNA = c("ALL","CLL","DLBC","LAML","LCML","MM",
                    "BLCA","KIRC","PRAD",
                    "BRCA","OV","UCEC",
                    "CHOL","COAD/READ","ESCA","LIHC","PAAD","STAD",
                    "GBM","LGG","MB","NB",
                    "LUAD","LUSC","MESO","SCLC",
                    "HNSC","SARC","SKCM","THCA")
color1=list(miRNA_group=color_cluster_miRNA,TCGA_disease=color_type_miRNA)
names(color1[[1]])=rownames(unique(miRNA_cluster$cluster_output))

miRNA=data.frame(unique(miRNA_result$label))
miRNA_order=factor(unique(miRNA_result$label),levels = TCGA_label_miRNA)
miRNA=miRNA[order(miRNA_order),]
names(color1[[2]])=as.vector(miRNA)

####miRNA Heatmap####
library(pheatmap)
library(RColorBrewer) 
annotation_col_miRNA = data.frame(miRNA_group = miRNA_cluster$cluster_output,TCGA_disease=miRNA_result$label[rownames(miRNA_cluster$cluster_output)])
rownames(annotation_col_miRNA) = rownames(miRNA_cluster$cluster_output)
annotation_col_miRNA = annotation_col_miRNA[order(annotation_col_miRNA[,1],annotation_col_miRNA[,2]),] 
miRNA_result$Data[which(miRNA_result$Data>10)]=10 
miRNA_result$Data[which(miRNA_result$Data<2)]=2 
pheatmap(miRNA_result$Data[miRNA_cluster$signature_result,rownames(annotation_col_miRNA)],cluster_cols = F,cluster_rows = T,
         annotation_col = annotation_col_miRNA,show_colnames = F,show_rownames = F,
         annotation_colors=color1,
         color = colorRampPalette(colors = c("blue","white","red"))(100))

####Color of CNV_cancer types####
library(RColorBrewer)
color_type_CNV=c(brewer.pal(9,"Reds")[-c(1,2,9)],brewer.pal(9,"Greens")[-c(1,2,3,7,8,9)],brewer.pal(9,"PuRd")[-c(1,2,4,7,8,9)],brewer.pal(9,"Blues")[-c(1,2,3,9)],brewer.pal(9,"Oranges")[-c(1,2,7,8,9)],brewer.pal(9,"Purples")[-c(1,2,3,8,9)],brewer.pal(8,"Set2")[-c(2,3,4,8)])
color_cluster_CNV=brewer.pal(12,"Set3")[-c(2,9)]
TCGA_label_CNV = c("ALL","CLL","DLBC","LAML","LCML","MM",
                     "BLCA","KIRC","PRAD",
                     "BRCA","OV","UCEC",
                     "COAD/READ","ESCA","LIHC","PAAD","STAD",
                     "GBM","LGG","MB","NB",
                     "LUAD","LUSC","MESO","SCLC",
                     "HNSC","SARC","SKCM","THCA")
color1=list(CNV_group=color_cluster_CNV,TCGA_disease=color_type_CNV)
names(color1[[1]])=rownames(unique(CNV_cluster$cluster_output))

CNV=data.frame(unique(CNV_result$label))
CNV_order=factor(unique(CNV_result$label),levels = TCGA_label_CNV)
CNV=CNV[order(CNV_order),]
names(color1[[2]])=as.vector(CNV)

####CNV Heatmap####
library(pheatmap)
library(RColorBrewer)
annotation_col_CNV = data.frame(CNV_group = CNV_cluster$cluster_output,TCGA_disease=CNV_result$label[rownames(CNV_cluster$cluster_output)])
rownames(annotation_col_CNV) = rownames(CNV_cluster$cluster_output)
annotation_col_CNV = annotation_col_CNV[order(annotation_col_CNV[,1],annotation_col_CNV[,2]),] 
CNV_result$Data[which(CNV_result$Data>1)]=1
CNV_result$Data[which(CNV_result$Data<(-1))]=-1
pheatmap(CNV_result$Data[CNV_cluster$signature_result,rownames(annotation_col_CNV)],cluster_cols = F,cluster_rows = T,
         annotation_col = annotation_col_CNV,show_colnames = F,show_rownames = T,
         annotation_colors=color1,
         color = colorRampPalette(colors = c("blue","white","red"))(100))

####Color of methy_cancer types####
library(RColorBrewer)
color_type_methy=c(brewer.pal(9,"Reds")[-c(1,2,9)],brewer.pal(9,"Greens")[-c(1,2,3,7,8,9)],brewer.pal(9,"PuRd")[-c(1,2,4,7,8,9)],brewer.pal(9,"Blues")[-c(1,2,9)],brewer.pal(9,"Oranges")[-c(1,2,7,8,9)],brewer.pal(9,"Purples")[-c(1,2,3,8,9)],brewer.pal(8,"Set2")[-c(2,3,4,8)])
color_cluster_methy=brewer.pal(12,"Set3")[-c(2,9)]
TCGA_label_methy = c("ALL","CLL","DLBC","LAML","LCML","MM",
                   "BLCA","KIRC","PRAD",
                   "BRCA","OV","UCEC",
                   "CHOL","COAD/READ","ESCA","LIHC","PAAD","STAD",
                   "GBM","LGG","MB","NB",
                   "LUAD","LUSC","MESO","SCLC",
                   "HNSC","SARC","SKCM","THCA")
color1=list(methy_group=color_cluster_methy,TCGA_disease=color_type_methy)
names(color1[[1]])=rownames(unique(methy_cluster$cluster_output))

methy=data.frame(unique(methy_result$label))
methy_order=factor(unique(methy_result$label),levels = TCGA_label_methy)
methy=methy[order(methy_order),]
names(color1[[2]])=as.vector(methy)

####methy Heatmap####
library(pheatmap)
library(RColorBrewer)
annotation_col_methy = data.frame(methy_group = methy_cluster$cluster_output,TCGA_disease=methy_result$label[rownames(methy_cluster$cluster_output)])
rownames(annotation_col_methy) = rownames(methy_cluster$cluster_output)
annotation_col_methy = annotation_col_methy[order(annotation_col_methy[,1],annotation_col_methy[,2]),] 
pheatmap(methy_result$Data[methy_cluster$signature_result,rownames(annotation_col_methy)],cluster_cols = F,cluster_rows = T,
         annotation_col = annotation_col_methy,show_colnames = F,show_rownames = F,
         annotation_colors=color1,
         color = colorRampPalette(colors = c("blue","white","red"))(100))

####Color of RPPA_cancer types####
library(RColorBrewer)
color_type_RPPA=c(brewer.pal(9,"Reds")[-c(1,2,9)],brewer.pal(9,"Greens")[-c(1,2,3,7,8,9)],brewer.pal(9,"PuRd")[-c(1,2,4,7,8,9)],brewer.pal(9,"Blues")[-c(1,2,9)],brewer.pal(9,"Oranges")[-c(1,2,7,8,9)],brewer.pal(9,"Purples")[-c(1,2,3,8,9)],brewer.pal(8,"Set2")[-c(2,3,4,8)])
color_cluster_RPPA=brewer.pal(12,"Set3")[-c(2,9)]
TCGA_label_RPPA = c("ALL","CLL","DLBC","LAML","LCML","MM",
                     "BLCA","KIRC","PRAD",
                     "BRCA","OV","UCEC",
                     "CHOL","COAD/READ","ESCA","LIHC","PAAD","STAD",
                     "GBM","LGG","MB","NB",
                     "LUAD","LUSC","MESO","SCLC",
                     "HNSC","SARC","SKCM","THCA")
color1=list(RPPA_group=color_cluster_RPPA,TCGA_disease=color_type_RPPA)
names(color1[[1]])=rownames(unique(RPPA_cluster$cluster_output))

RPPA=data.frame(unique(RPPA_result$label))
RPPA_order=factor(unique(RPPA_result$label),levels = TCGA_label_RPPA)
RPPA=RPPA[order(RPPA_order),]
names(color1[[2]])=as.vector(RPPA)

####RPPA Heatmap####
library(pheatmap)
library(RColorBrewer)
annotation_col_RPPA = data.frame(RPPA_group = RPPA_cluster$cluster_output,TCGA_disease=RPPA_result$label[rownames(RPPA_cluster$cluster_output)])
rownames(annotation_col_RPPA) = rownames(RPPA_cluster$cluster_output)
annotation_col_RPPA = annotation_col_RPPA[order(annotation_col_RPPA[,1],annotation_col_RPPA[,2]),]
RPPA_result$Data[which(RPPA_result$Data>2)]=2
RPPA_result$Data[which(RPPA_result$Data<(-2))]=-2
pheatmap(RPPA_result$Data[RPPA_cluster$signature_result,rownames(annotation_col_RPPA)],cluster_cols = F,cluster_rows = T,
         annotation_col = annotation_col_RPPA,show_colnames = F,show_rownames = F,
         annotation_colors=color1,
         color = colorRampPalette(colors = c("blue","white","red"))(100))

#Statistics
mRNA_stat = Statistic(mRNA_cluster$cluster_number,annotation_col_mRNA)
miRNA_stat = Statistic(miRNA_cluster$cluster_number,annotation_col_miRNA)
CNV_stat = Statistic(CNV_cluster$cluster_number,annotation_col_CNV)
methy_stat = Statistic(methy_cluster$cluster_number,annotation_col_methy)
RPPA_stat = Statistic(RPPA_cluster$cluster_number,annotation_col_RPPA)


####Write result####


