####Function: calculate the percentage of cancer types in each cluster####
library(RColorBrewer)
Ratio = function(Data)
{
  Data[,2]=as.numeric(Data[,2]) 
  ratio_matrix=matrix(0, nrow = length(unique(Data[,2])), ncol = length(unique(Data[,3]))) 
  rownames(ratio_matrix)=paste("cluster",c(1:length(unique(Data[,2]))),sep = " ")
  colnames(ratio_matrix)=sort(unique(Data[,3]))
  
  for (i in 1:length(unique(Data[,2])))
  {
    for (j in 1:length(unique(Data[,3]))) 
    {
      ratio_matrix[i,j]=length(which(Data[which(Data[,2]==i),3]==colnames(ratio_matrix)[j]))/length(which(Data[,2]==i))
    }
  }
  rm(i,j)
  return(ratio_matrix)
}

####Function: calculate the p-value of cancer types in each cluster####
p_value = function(Data)
{
  Data[,2]=as.numeric(Data[,2])
  p_matrix=matrix(0, nrow = length(unique(Data[,2])), ncol = length(unique(Data[,3])))
  rownames(p_matrix)=paste("cluster",c(1:length(unique(Data[,2]))),sep = " ")
  colnames(p_matrix)=sort(unique(Data[,3]))
  
  for (i in 1:length(unique(Data[,2])))
  {
    for (j in 1:length(unique(Data[,3]))) 
    {
      p_matrix[i,j]=length(which(Data[which(Data[,2]==i),3]==colnames(p_matrix)[j]))
    }
  }
  
  pvalue_matrix=p_matrix
  for (i in 1:length(unique(Data[,2])))
  {
    for (j in 1:length(unique(Data[,3])))
    {
      N= length(Data[,1])
      M= sum(p_matrix[i,])
      n= sum(p_matrix[,j])
      k= p_matrix[i,j]
      pvalue_matrix[i,j]=phyper(k-1,M, N-M, n, lower.tail=FALSE) 
      rm(M,N,n,k)
    }
  }
  if(0)
  {
    pvalue_matrix=matrix(p.adjust(pvalue_matrix,method = "BH"),nrow = length(unique(Data[,2]))) 
    rownames(pvalue_matrix)=rownames(p_matrix)
    colnames(pvalue_matrix)=colnames(p_matrix)
  }
  rm(i,j,p_matrix)
  return(pvalue_matrix)
}

####Function: calculate the Jaccard of cancer types in each cluster####
Jaccard = function(Data)
{
  Data[,2]=as.numeric(Data[,2])
  J_matrix=matrix(0, nrow = length(unique(Data[,2])), ncol = length(unique(Data[,3])))
  rownames(J_matrix)=paste("cluster",c(1:length(unique(Data[,2]))),sep = " ")
  colnames(J_matrix)=sort(unique(Data[,3]))
  
  for (i in 1:length(unique(Data[,2])))
  {
    for (j in 1:length(unique(Data[,3]))) 
    {
      J_matrix[i,j]=length(which(Data[which(Data[,2]==i),3]==colnames(J_matrix)[j]))
    }
  }
  
  Jaccard_matrix=J_matrix
  for (i in 1:length(unique(Data[,2])))
  {
    for (j in 1:length(unique(Data[,3]))) 
    {
      Jaccard_matrix[i,j]=J_matrix[i,j]/(sum(J_matrix[i,])+sum(J_matrix[,j])-J_matrix[i,j])
    }
  }
  rm(i,j,J_matrix)
  return(Jaccard_matrix)
}

####Main####
####Read omics data####

####calculate ratio and p_value####
ratio_mRNA = Ratio(mRNA_temp)
p_mRNA = p_value(mRNA_temp)
Jaccard_mRNA = Jaccard(mRNA_temp)

ratio_miRNA = Ratio(miRNA_temp)
p_miRNA = p_value(miRNA_temp)
Jaccard_miRNA = Jaccard(miRNA_temp)

ratio_CNV = Ratio(CNV_temp)
p_CNV = p_value(CNV_temp)
Jaccard_CNV = Jaccard(CNV_temp)

ratio_methy = Ratio(methy_temp)
p_methy = p_value(methy_temp)
Jaccard_methy = Jaccard(methy_temp)

ratio_RPPA = Ratio(RPPA_temp)
p_RPPA = p_value(RPPA_temp)
Jaccard_RPPA = Jaccard(RPPA_temp)

####Heatmap####
####mRNA Heatmap####
library(pheatmap)
library(RColorBrewer)

pheatmap(ratio_mRNA,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         main = "mRNA: Ratio of cancer types",angle_col=45)

p_mRNA_temp=-log10(p_mRNA)
p_mRNA_temp[which(p_mRNA_temp>10)]=10
pheatmap(p_mRNA_temp,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         color = colorRampPalette(c("GhostWhite","firebrick3"))(1000),
         main = "Dominated cancer types of mRNA clusters",angle_col=45)

pheatmap(Jaccard_mRNA,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         main = "mRNA: Jaccard of cancer types",angle_col=45)

####miRNA Heatmap####

pheatmap(ratio_miRNA,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         main = "miRNA: Ratio of cancer types",angle_col=45)

p_miRNA_temp=-log10(p_miRNA)
p_miRNA_temp[which(p_miRNA_temp>10)]=10
pheatmap(p_miRNA_temp,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         color = colorRampPalette(c("GhostWhite","firebrick3"))(1000),
         main = "Dominated cancer types of miRNA clusters",angle_col=45)

pheatmap(Jaccard_miRNA,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         main = "miRNA: Jaccard of cancer types",angle_col=45)

####CNV Heatmap####

pheatmap(ratio_CNV,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         main = "CNV: Ratio of cancer types",angle_col=45)

p_CNV_temp=-log10(p_CNV)
p_CNV_temp[which(p_CNV_temp>10)]=10
pheatmap(p_CNV_temp,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         color = colorRampPalette(c("GhostWhite","firebrick3"))(1000),
         main = "Dominated cancer types of CNV clusters",angle_col=45)

pheatmap(Jaccard_CNV,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         main = "CNV: Jaccard of cancer types",angle_col=45)

####methy Heatmap####

pheatmap(ratio_methy,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         main = "methy: Ratio of cancer types",angle_col=45)

p_methy_temp=-log10(p_methy)
p_methy_temp[which(p_methy_temp>10)]=10
pheatmap(p_methy_temp,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         color = colorRampPalette(c("GhostWhite","firebrick3"))(1000),
         main = "Dominated cancer types of METHY clusters",angle_col=45)

pheatmap(Jaccard_methy,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         main = "methy: Jaccard of cancer types",angle_col=45)

####RPPA Heatmap####

pheatmap(ratio_RPPA,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         main = "RPPA: Ratio of cancer types",angle_col=45)

p_RPPA_temp=-log10(p_RPPA)
p_RPPA_temp[which(p_RPPA_temp>10)]=10
pheatmap(p_RPPA_temp,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         color = colorRampPalette(c("GhostWhite","firebrick3"))(1000),
         main = "Dominated cancer types of RPPA clusters",angle_col=45)

pheatmap(Jaccard_RPPA,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         main = "RPPA: Jaccard of cancer types",angle_col=45)

####Write results####


