library(SNFtool)
library(CancerSubtypes)

####Main####
####Read omic data####

####Overlapped sample names####
label_temp = label_temp[-which(label_temp[,8]=="UNABLE TO CLASSIFY"),] 
label_temp = label_temp[-which(is.na(label_temp[,8])),]

ALL_names = NULL
ALL_names = label_temp[,1]
ALL_names = intersect(ALL_names, colnames(mRNA_temp))
ALL_names = intersect(ALL_names, colnames(miRNA_temp))
ALL_names = intersect(ALL_names, colnames(CNV_temp))
ALL_names = intersect(ALL_names, colnames(methy_temp))
ALL_names = intersect(ALL_names, colnames(RPPA_temp))

####Mapped samples####
label_new = label_temp[which(label_temp[,1] %in% ALL_names),]
mRNA_temp = mRNA_temp[, ALL_names]
miRNA_temp = miRNA_temp[, ALL_names]
CNV_temp = CNV_temp[, ALL_names]
methy_temp = methy_temp[, ALL_names]
RPPA_temp = RPPA_temp[, ALL_names]

####Data normalization####
mRNA_norm = data.normalization(mRNA_temp, type = "feature_Median", log2 = FALSE)
miRNA_norm = data.normalization(miRNA_temp, type = "feature_Median", log2 = FALSE)
CNV_norm = data.normalization(CNV_temp, type = "feature_Median", log2 = FALSE)
methy_norm = data.normalization(methy_temp, type = "feature_Median", log2 = FALSE)
RPPA_norm = data.normalization(RPPA_temp, type = "feature_Median", log2 = FALSE)

####Multi-omics cluster: method SNF-CC####
list_temp = list(mRNA_norm,miRNA_norm,CNV_norm,methy_norm,RPPA_norm)
result = ExecuteSNF.CC(list_temp, clusterNum=18, K=20, alpha=0.5, t=20,
                     maxK = 30, pItem = 0.8,reps=500, 
                     finalLinkage ="average")
group = as.matrix(result$group)
sil = silhouette_SimilarityMatrix(result$group, result$distanceMatrix)
plot(sil)


mRNA_t = t(mRNA_norm)
miRNA_t = t(miRNA_norm)
CNV_t = t(CNV_norm)
methy_t = t(methy_norm)
RPPA_t = t(RPPA_norm)
list_t = list(mRNA_t,miRNA_t,CNV_t,methy_t,RPPA_t)
NMI_scores = rankFeaturesByNMI(list_t, result$distanceMatrix) 

NMI_matrix = matrix(data = NA,nrow = length(NMI_scores[[1]][[1]])+length(NMI_scores[[1]][[2]])+
                      length(NMI_scores[[1]][[3]])+length(NMI_scores[[1]][[4]])+length(NMI_scores[[1]][[5]]),
                    ncol = 2)
colnames(NMI_matrix) = c("score","rank")
mRNA_names = as.numeric(rownames(mRNA_temp))
miRNA_names = rownames(miRNA_temp)
CNV_names = rownames(CNV_temp)
methy_names = rownames(methy_temp)
RPPA_names = rownames(RPPA_temp)
all_f_names = c(mRNA_names,miRNA_names,CNV_names,methy_names,RPPA_names)
rownames(NMI_matrix) = all_f_names

NMI_matrix[,1] = unlist(NMI_scores[[1]])
NMI_matrix[,2] = rank(NMI_matrix[,1])

NMI_matrix = NMI_matrix[order(NMI_matrix[,2],decreasing=T),]
NMI_matrix_20 = NMI_matrix[c(1:13453),]

mRNA_ratio = length(which(rownames(NMI_matrix_20) %in% mRNA_names))/length(mRNA_names)
miRNA_ratio = length(which(rownames(NMI_matrix_20) %in% miRNA_names))/length(miRNA_names)
CNV_ratio = length(which(rownames(NMI_matrix_20) %in% CNV_names))/length(CNV_names)
methy_ratio = length(which(rownames(NMI_matrix_20) %in% methy_names))/length(methy_names)
RPPA_ratio = length(which(rownames(NMI_matrix_20) %in% RPPA_names))/length(RPPA_names)

#The hist of omics contribution of each data type
library(ggplot2)
omics_ratio = matrix(data = NA, ncol = 2, nrow = 5)
colnames(omics_ratio) = c("omics","feature")
omics_ratio[,1] = c("mRNA","miRNA","CNV","methy","RPPA")
omics_ratio[,2] = c(mRNA_ratio,miRNA_ratio,CNV_ratio,methy_ratio,RPPA_ratio)
omics_ratio = as.data.frame(omics_ratio)
omics_ratio[,2] = as.numeric(as.character(omics_ratio[,2])) 
x=factor(omics_ratio[,1],levels = c("mRNA","miRNA","CNV","methy","RPPA")) 
ggplot(omics_ratio,aes(x=x,y=feature)) + geom_bar(stat="identity",fill="lightblue",colour="black",width=0.3) +
  labs(x="omics data type",y="top features/all features",title = "the top 20% NMI features") +
  theme_set(theme_bw()) + theme(panel.grid=element_line(colour=NA))


####Heatmap####
library(pheatmap)
library(RColorBrewer)

group_label = cbind(group,label_new)
group_label = group_label[,-c(2:8)]
colnames(group_label)=c("SNF_CC","TCGA_disease")
group_label = as.data.frame(group_label)
group_label[,1] = as.numeric(group_label[,1])
group_label = group_label[order(group_label[,1],group_label[,2]),] 

####Color of cancer types####
library(RColorBrewer)
color_type=c(brewer.pal(9,"Reds")[-c(1,2,9)],brewer.pal(9,"Greens")[-c(1,2,3,7,8,9)],brewer.pal(9,"PuRd")[-c(1,2,4,7,8,9)],brewer.pal(9,"Blues")[-c(1,2,3,9)],brewer.pal(9,"Oranges")[-c(1,2,7,8,9)],brewer.pal(9,"Purples")[-c(1,2,3,8,9)],brewer.pal(8,"Set2")[-c(2,3,4,8)])
color_cluster=c(brewer.pal(9,"Purples")[7],brewer.pal(9,"Oranges")[3],brewer.pal(9,"Blues")[7],
                brewer.pal(9,"Reds")[3],brewer.pal(8,"Set2")[1],brewer.pal(8,"Set2")[5],
                brewer.pal(9,"PuRd")[3],brewer.pal(9,"Reds")[2],brewer.pal(9,"Reds")[7],
                brewer.pal(9,"Reds")[8],brewer.pal(9,"Blues")[4],brewer.pal(9,"Reds")[5],
                brewer.pal(9,"Purples")[4],brewer.pal(9,"Reds")[6],brewer.pal(8,"Set2")[6],
                brewer.pal(9,"Oranges")[6],brewer.pal(9,"Greens")[5],brewer.pal(9,"PuRd")[5])
TCGA = c("ALL","CLL","DLBC","LAML","LCML","MM",
                   "BLCA","KIRC","PRAD",
                   "BRCA","OV","UCEC",
                   "COAD/READ","ESCA","LIHC","PAAD","STAD",
                   "GBM","LGG","MB","NB",
                   "LUAD","LUSC","MESO","SCLC",
                   "HNSC","SARC","SKCM","THCA")
color1=list(SNF_CC=color_cluster,TCGA_disease=color_type)
names(color1[[1]])=unique(group_label[,1])

SNF_group=data.frame(unique(group_label[,2]))
SNF_group_order=factor(unique(group_label[,2]),levels = TCGA)
SNF_group=SNF_group[order(SNF_group_order),]
names(color1[[2]])=as.vector(SNF_group)

rownames(mRNA_temp)=as.numeric(rownames(mRNA_temp))
mRNA=mRNA_temp[rownames(NMI_matrix_20)[which(rownames(NMI_matrix_20) %in% mRNA_names)],]
miRNA=miRNA_temp[rownames(NMI_matrix_20)[which(rownames(NMI_matrix_20) %in% miRNA_names)],]
CNV=CNV_temp[rownames(NMI_matrix_20)[which(rownames(NMI_matrix_20) %in% CNV_names)],]
methy=methy_temp[rownames(NMI_matrix_20)[which(rownames(NMI_matrix_20) %in% methy_names)],]
RPPA=RPPA_temp[rownames(NMI_matrix_20)[which(rownames(NMI_matrix_20) %in% RPPA_names)],]

####Write result####

#mRNA heatmap
mRNA[which(mRNA>10)]=10
pheatmap(mRNA[,rownames(group_label)],cluster_cols = F,cluster_rows = T,
         annotation_col = group_label,show_colnames = F,show_rownames = F,
         annotation_colors=color1,
         color = colorRampPalette(colors = c("blue","white","red"))(100))
#miRNA heatmap
miRNA[which(miRNA>10)]=10
miRNA[which(miRNA<2)]=2
pheatmap(miRNA[,rownames(group_label)],cluster_cols = F,cluster_rows = T,
         annotation_col = group_label,show_colnames = F,show_rownames = F,
         annotation_colors=color1,
         color = colorRampPalette(colors = c("blue","white","red"))(100))
#CNV heatmap
CNV_temp[which(CNV_temp>1)]=1
CNV_temp[which(CNV_temp<(-1))]=-1
pheatmap(CNV_temp[,rownames(group_label)],cluster_cols = F,cluster_rows = T,
         annotation_col = group_label,show_colnames = F,show_rownames = T,
         annotation_colors=color1,
         color = colorRampPalette(colors = c("blue","white","red"))(100))
#methy heatmap
pheatmap(methy[,rownames(group_label)],cluster_cols = F,cluster_rows = T,
         annotation_col = group_label,show_colnames = F,show_rownames = F,
         annotation_colors=color1,
         color = colorRampPalette(colors = c("blue","white","red"))(100))
#RPPA heatmap
RPPA[which(RPPA>2)]=2
RPPA[which(RPPA<(-2))]=-2
pheatmap(RPPA[,rownames(group_label)],cluster_cols = F,cluster_rows = T,
         annotation_col = group_label,show_colnames = F,show_rownames = F,
         annotation_colors=color1,
         color = colorRampPalette(colors = c("blue","white","red"))(100))
