####Function: calculate the percentage of cancer types in each cluster####
library(RColorBrewer)
Ratio = function(Data)
{
  Data[,2]=as.numeric(Data[,2])
  ratio_matrix=matrix(0, nrow = length(unique(Data[,2])), ncol = length(unique(Data[,3]))) 
  rownames(ratio_matrix)=paste("cluster",c(1:length(unique(Data[,2]))),sep = " ")
  colnames(ratio_matrix)=sort(unique(Data[,3]))
  
  for (i in 1:length(unique(Data[,2])))
  {
    for (j in 1:length(unique(Data[,3]))) 
    {
      ratio_matrix[i,j]=length(which(Data[which(Data[,2]==i),3]==colnames(ratio_matrix)[j]))/length(which(Data[,2]==i))
    }
  }
  rm(i,j)
  return(ratio_matrix)
}

####Function: calculate the p-value of cancer types in each cluster####
p_value = function(Data)
{
  Data[,2]=as.numeric(Data[,2])
  p_matrix=matrix(0, nrow = length(unique(Data[,2])), ncol = length(unique(Data[,3])))
  rownames(p_matrix)=paste("cluster",c(1:length(unique(Data[,2]))),sep = " ")
  colnames(p_matrix)=sort(unique(Data[,3]))
  
  for (i in 1:length(unique(Data[,2])))
  {
    for (j in 1:length(unique(Data[,3]))) 
    {
      p_matrix[i,j]=length(which(Data[which(Data[,2]==i),3]==colnames(p_matrix)[j]))
    }
  }
  
  pvalue_matrix=p_matrix
  for (i in 1:length(unique(Data[,2])))
  {
    for (j in 1:length(unique(Data[,3])))
    {
      N= length(Data[,1])
      M= sum(p_matrix[i,])
      n= sum(p_matrix[,j])
      k= p_matrix[i,j]
      pvalue_matrix[i,j]=phyper(k-1,M, N-M, n, lower.tail=FALSE) 
      rm(M,N,n,k)
    }
  }
  if(0)
  {
    pvalue_matrix=matrix(p.adjust(pvalue_matrix,method = "BH"),nrow = length(unique(Data[,2]))) 
    rownames(pvalue_matrix)=rownames(p_matrix)
    colnames(pvalue_matrix)=colnames(p_matrix)
  }
  rm(i,j,p_matrix)
  return(pvalue_matrix)
}

####Function: calculate the Jaccard of cancer types in each cluster####
Jaccard = function(Data)
{
  Data[,2]=as.numeric(Data[,2])
  J_matrix=matrix(0, nrow = length(unique(Data[,2])), ncol = length(unique(Data[,3])))
  rownames(J_matrix)=paste("cluster",c(1:length(unique(Data[,2]))),sep = " ")
  colnames(J_matrix)=sort(unique(Data[,3]))
  
  for (i in 1:length(unique(Data[,2])))
  {
    for (j in 1:length(unique(Data[,3]))) 
    {
      J_matrix[i,j]=length(which(Data[which(Data[,2]==i),3]==colnames(J_matrix)[j]))
    }
  }

  Jaccard_matrix=J_matrix
  for (i in 1:length(unique(Data[,2])))
  {
    for (j in 1:length(unique(Data[,3]))) 
    {
      Jaccard_matrix[i,j]=J_matrix[i,j]/(sum(J_matrix[i,])+sum(J_matrix[,j])-J_matrix[i,j])
    }
  }
  rm(i,j,J_matrix)
  return(Jaccard_matrix)
}

####Main####
####Read omics data####

colnames(multi_temp)=c("sample","SNF_CC","TCGA_label")

####calculate ratio and p_value####
ratio_multi = Ratio(multi_temp)
p_multi = p_value(multi_temp)
Jaccard_multi = Jaccard(multi_temp)

####Heatmap####
####mRNA Heatmap####
library(pheatmap)
library(RColorBrewer)

pheatmap(ratio_multi,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         main = "Multi omics: Ratio of cancer types",angle_col=45)

p_multi_temp=-log10(p_multi)
p_multi_temp[which(p_multi_temp>10)]=10
pheatmap(p_multi_temp,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         color = colorRampPalette(c("GhostWhite","firebrick3"))(1000),
         main = "Dominated cancer types of clusters in integrated multiple omics  clustering",angle_col=45)

pheatmap(Jaccard_multi,cluster_cols = F,cluster_rows = F,display_numbers = TRUE,
         main = "Multi omics: Jaccard of cancer types",angle_col=45)

####Write results####

